<html>
  <head><title>[% title %]</title></head>
  <body>
    <ul>
      [% SECTION post %]
        <li>
            <h3>[% title %]</h3>
            <span>[% date %]</span>
        </li>
      [% END %]
    </ul>
  </body>
</html>
